#!/usr/bin/python
#********************************************************************************
# Licensed Materials - Property of IBM
# IBM Quick Deployer
# (c) Copyright IBM Corporation 2015. All Rights Reserved.
#
# U.S. Government Users Restricted Rights:
# Use, duplication or disclosure restricted by GSA ADP Schedule
# Contract with IBM Corp.
#
#********************************************************************************
#############################################################################
# Name: processTagsLib.py
#
# Purpose: 
#    1. extract the TAGS from a "server",
#       create a new element [hostname]:[csv list of tags]
#       create or add to UCD environment property {globalTagList} and populate with the new element
#    2. create localTagList and localHostname
#       add or update their entries in the env.properties file
#
###################################################

import sys
import os
import shutil
import subprocess
import platform
import json
import inspect
import time
import socket

# local file
import globalProps


####################################################
# ===================== FUNCTIONS =====================
####################################################



#########################################
#
# Function determineOS()
#
#####################################

def determineOS():

    if  platform.system().startswith('Windows'):
        globalProps.bOSWINDOWS = "True"
    else:
        globalProps.bOSWINDOWS = "False"
    print "running on [" + platform.system() + "] operating system"

#########################################
#
# Function setEnvProps()
#
#####################################

def setEnvProps():

    if globalProps.bOSWINDOWS == "True":
        # quote to allow for "Program File"
        globalProps.UCDCLIENT_EXE = """" + globalProps.AGENT_HOME + "\opt\udclient\udclient"""
        globalProps.LOCALSERVERPROPERTYFILE = globalProps.LOCALSERVERPROPERTYFOLDER + "\\" + "env.properties"
    else:
        # linux
        globalProps.UCDCLIENT_EXE = globalProps.AGENT_HOME + "/opt/udclient/udclient"
        globalProps.LOCALSERVERPROPERTYFILE = globalProps.LOCALSERVERPROPERTYFOLDER + "/" + "env.properties"
    
    print "LOCALSERVERPROPERTYFILE [" + globalProps.LOCALSERVERPROPERTYFILE  + "]"
    
    print "UCDCLIENT_EXE [" + globalProps.UCDCLIENT_EXE  + "]"
    if os.path.exists(globalProps.UCDCLIENT_EXE):
        print "file exists"
    else:
        print "ERROR: file does not exist"
        sys.exit(1)
    
    if globalProps.bOSWINDOWS == "False":
        # if linux make it executable
        subprocess.Popen(["ls", "-la", globalProps.UCDCLIENT_EXE])
        print "ensure file is executable"
        subprocess.Popen(["chmod", "u+x", globalProps.UCDCLIENT_EXE])
        time.sleep(5)
        subprocess.Popen(["ls", "-la", globalProps.UCDCLIENT_EXE])



#########################################
#
# Function checkExists(errString)
#
# check the stdErr returned by a UCD api call
# returns 
# -- "exists" if stdErr is empty
# -- one of the following depending on the errString
# ---- "No resource"
# ---- "No agent"
# ---- "No environment"
# ---- "No application"
# ---- "Property not found"
# ---- "error"
#
#####################################
def checkExists(errString):

    if errString and errString.strip():

        if errString.find("No resource with id/name") == 0:
            rc = "No resource"
        elif errString.find("No agent with id/name") == 0:
            rc = "No agent"
        elif errString.find("No environment") == 0:
            rc = "No environment"
        elif errString.find("No application") == 0:
            rc = "No application"
        elif errString.find("Property") == 0:
            if errString.find("not found") > 0:
                rc = "Property not found"
            else:
                rc = "error"
        else:
            rc = "error"

    else:
        rc = "exists"

    print "[checkExists] [" + rc + "]"

    return rc


#########################################
#
# Function checkStdErr(errString)
#
# check the stdErr returned by a UCD api call
# FAIL and EXIT if it contains content
# print 2 levels of calling function tree to aid debug
#
#####################################
def checkStdErr(errString):

    thisFunctionName = inspect.stack()[0][3]
    parentFunctionName = inspect.stack()[1][3]
    grandParentFunctionName = inspect.stack()[2][3]
    callingTree = "[" + thisFunctionName + "] [" + parentFunctionName + "] [" + grandParentFunctionName + "]"
    if errString and errString.strip():
        print "ERROR: function tree : " + callingTree
        print "error message: [" + errString + "]"
        sys.exit(1)
    else:
        if globalProps.bDEBUGFLAG == "True":
            print "OK: error string empty! " + callingTree



#########################################
#
# Function getResource()
#
# getResource info using the ucdclient api
#
#####################################

def getResource(resourceName):

    print "get resource [" + resourceName + "]"
    tempCmd = globalProps.UCDCLIENT_EXE
    tempArgs = " -weburl " + globalProps.UCDSERVER + " -authtoken " + globalProps.AUTH_TOKEN + " getResource  -resource " + resourceName

    # execute command
    if globalProps.bDEBUGFLAG == "True":
        print "DEBUG: tempCmd [" + tempCmd + "]"
        print "DEBUG: tempArgs [" + tempArgs + "]"
    
    p = subprocess.Popen([tempCmd, "-weburl", globalProps.UCDSERVER, "-authtoken", globalProps.AUTH_TOKEN, "getResource", "-resource", resourceName], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    tempOut, tempErr = p.communicate()
    
    if globalProps.bDEBUGFLAG == "True":
        print "DEBUG: tempOut [" + tempOut + "]"
        print "DEBUG: tempErr [" + tempErr + "]"

    # remove trailing \t\r\n and whitspace
    tempErr = tempErr.rstrip()
    # check resource exists
    rc = checkExists(tempErr)
    if "exists" == rc:
        # remove trailing \t\r\n and whitspace
        resourceInfo = tempOut.rstrip()
    elif "No resource" == rc:
        # cannot find the resouce
        checkStdErr(tempErr)
    else:
        # something else went wrong
        checkStdErr(tempErr)
    
    return resourceInfo


#########################################
#
# Function testAgentConnection()
#
# test the agent/server connection using the ucdclient api
#
#####################################

def testAgentConnection():

    print "test agent connection"
    tempCmd = globalProps.UCDCLIENT_EXE
    tempArgs = " -weburl " + globalProps.UCDSERVER + " -authtoken " + globalProps.AUTH_TOKEN + " testAgent -agent " + globalProps.AGENT_ID

    # execute command
    if globalProps.bDEBUGFLAG == "True":
        print "DEBUG: tempCmd [" + tempCmd + "]"
        print "DEBUG: tempArgs [" + tempArgs + "]"
    
    p = subprocess.Popen([tempCmd, "-weburl", globalProps.UCDSERVER, "-authtoken", globalProps.AUTH_TOKEN, "testAgent", "-agent", globalProps.AGENT_ID], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    tempOut, tempErr = p.communicate()
    
    if globalProps.bDEBUGFLAG == "True":
        print "DEBUG: tempOut [" + tempOut + "]"
        print "DEBUG: tempErr [" + tempErr + "]"   

    if tempOut and tempOut.strip():
        if tempOut.find("Authentication failure") == 0:
            errMsg = "Authentication failure"
        elif tempOut.find("ConnectException") >= 0:
            errMsg = "Connect Exception"
        elif tempOut.find("UnknownHostException") >= 0:
            errMsg = "Unknown Host Exception"
        elif tempOut.find("[OK]") >= 0:
            if tempOut.count("[OK]") == 2:
                errMsg = "agent connected"
            else:
                errMsg = "unexpected error1"
        else:
            errMsg = "unexpected error2"

    else:
        errMsg = "unexpected error3"

    if "agent connected" == errMsg:
        print "agent connected"
    else:
        print "ERROR: agent connection error [" + errMsg + "]"
        print "STDOUT [" + tempOut.rstrip() + "]"
        print "STDERR [" + tempErr.rstrip() + "]"
        sys.exit(1)


#########################################
#
# Function getHostname()
#
# get the server hostname direct from the OS
#
#####################################

def getLinuxHostname():

    print "get server hostname"
    p = subprocess.Popen(["hostname", "-f"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    tempOut, tempErr = p.communicate()
    
    if globalProps.bDEBUGFLAG == "True":
        print "DEBUG: tempOut [" + tempOut + "]"
        print "DEBUG: tempErr [" + tempErr + "]"   
    
    # remove trailing \t\r\n and whitspace
    tempErr = tempErr.rstrip()
    # check resource exists
    rc = checkExists(tempErr)
    if "exists" == rc:
        # remove trailing \t\r\n and whitspace
        tempHostname = tempOut.rstrip()
    elif "Property not found" == rc:
        tempHostname = rc
    else:
        # something went wrong
        checkStdErr(tempErr)
    
    return tempHostname


#########################################
#
# Function getHostname()
#
# get the server hostname via the socket class
#
#####################################

def getHostname():

    print "get server hostname from socket"
    tempHostname = socket.getfqdn()
    
    if globalProps.bDEBUGFLAG == "True":
        print "DEBUG: tempHostname [" + tempHostname + "]"
    
    # remove trailing \t\r\n and whitespace
    tempHostname = tempHostname.rstrip()
    
    return tempHostname


#########################################
#
# Function getAgentProperty(tempProperty)
#
# getAgentProperty info using the ucdclient api
#
#####################################

def getAgentProperty(tempProperty):

    print "get agent property [" + tempProperty + "]"
    tempCmd = globalProps.UCDCLIENT_EXE
    tempArgs = " -weburl " + globalProps.UCDSERVER + " -authtoken " + globalProps.AUTH_TOKEN + " getAgentProperty -agent " + globalProps.AGENT_ID + " -name " + tempProperty

    # execute command
    if globalProps.bDEBUGFLAG == "True":
        print "DEBUG: tempCmd [" + tempCmd + "]"
        print "DEBUG: tempArgs [" + tempArgs + "]"
    
    p = subprocess.Popen([tempCmd, "-weburl", globalProps.UCDSERVER, "-authtoken", globalProps.AUTH_TOKEN, "getAgentProperty", "-agent", globalProps.AGENT_ID, "-name", tempProperty], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    tempOut, tempErr = p.communicate()
    
    if globalProps.bDEBUGFLAG == "True":
        print "DEBUG: tempOut [" + tempOut + "]"
        print "DEBUG: tempErr [" + tempErr + "]"   
    
    # remove trailing \t\r\n and whitespace
    tempErr = tempErr.rstrip()
    # check resource exists
    rc = checkExists(tempErr)
    if "exists" == rc:
        # remove trailing \t\r\n and whitespace
        agentProperty = tempOut.rstrip()
    elif "Property not found" == rc:
        agentProperty = rc
    else:
        # something went wrong
        checkStdErr(tempErr)
    
    return agentProperty


#########################################
#
# Function getEnvironmentProperty(tempProperty)
#
# getEnvironmentProperty info using the ucdclient api
#
#####################################

def getEnvironmentProperty(tempProperty):

    print "get environment property [" + tempProperty + "]"
    tempCmd = globalProps.UCDCLIENT_EXE
    tempArgs = " -weburl " + globalProps.UCDSERVER + " -authtoken " + globalProps.AUTH_TOKEN + " getEnvironmentProperty -application " + globalProps.APPLICATION_ID + " -environment " + globalProps.ENVIRONMENT_ID + " -name " + tempProperty

    # execute command
    if globalProps.bDEBUGFLAG == "True":
        print "DEBUG: tempCmd [" + tempCmd + "]"
        print "DEBUG: tempArgs [" + tempArgs + "]"
    
    p = subprocess.Popen([tempCmd, "-weburl", globalProps.UCDSERVER, "-authtoken", globalProps.AUTH_TOKEN, "getEnvironmentProperty", "-application", globalProps.APPLICATION_ID, "-environment", globalProps.ENVIRONMENT_ID, "-name", tempProperty], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    tempOut, tempErr = p.communicate()

    if globalProps.bDEBUGFLAG == "True":
        print "DEBUG: tempOut [" + tempOut + "]"
        print "DEBUG: tempErr [" + tempErr + "]"   
    
    # remove trailing \t\r\n and whitespace
    tempErr = tempErr.rstrip()
    # check resource exists
    rc = checkExists(tempErr)
    if "exists" == rc:
        # remove trailing \t\r\n and whitespace
        environmentProperty = tempOut.rstrip()
    elif "Property not found" == rc:
        environmentProperty = rc
    else:
        # something went wrong
        checkStdErr(tempErr)
    
    return environmentProperty


#########################################
#
# Function getAllEnvironmentProperties()
#
# getEnvironmentProperties using the ucdclient api
#
#####################################

def getAllEnvironmentProperties():

    print "get all environment properties"
    tempCmd = globalProps.UCDCLIENT_EXE
    tempArgs = " -weburl " + globalProps.UCDSERVER + " -authtoken " + globalProps.AUTH_TOKEN + " getEnvironmentProperties -application " + globalProps.APPLICATION_ID + " -environment " + globalProps.ENVIRONMENT_ID

    # execute command
    if globalProps.bDEBUGFLAG == "True":
        print "DEBUG: tempCmd [" + tempCmd + "]"
        print "DEBUG: tempArgs [" + tempArgs + "]"
    
    p = subprocess.Popen([tempCmd, "-weburl", globalProps.UCDSERVER, "-authtoken", globalProps.AUTH_TOKEN, "getEnvironmentProperties", "-application", globalProps.APPLICATION_ID, "-environment", globalProps.ENVIRONMENT_ID], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    tempOut, tempErr = p.communicate()
    
    if globalProps.bDEBUGFLAG == "True":
        print "DEBUG: tempOut [" + tempOut + "]"
        print "DEBUG: tempErr [" + tempErr + "]"   
    
    # remove trailing \t\r\n and whitespace
    tempErr = tempErr.rstrip()
    # check resource exists
    rc = checkExists(tempErr)
    if "exists" == rc:
        # remove trailing \t\r\n and whitespace
        environmentProperties = tempOut.rstrip()
    elif "Property not found" == rc:
        environmentProperties = rc
    else:
        # something went wrong
        checkStdErr(tempErr)
    
    return environmentProperties


#########################################
#
# Function setUCDEnvironmentProperty(tempProperty, tempValue)
#
# setUCDEnvironmentProperty info using the ucdclient api
#
#####################################

def setUCDEnvironmentProperty(tempProperty, tempValue):

    print "set environment property [" + tempProperty + "] [" + tempValue + "]"
    tempCmd = globalProps.UCDCLIENT_EXE
    tempArgs = " -weburl " + globalProps.UCDSERVER + " -authtoken " + globalProps.AUTH_TOKEN + " setEnvironmentProperty -application " + globalProps.APPLICATION_ID + " -environment " + globalProps.ENVIRONMENT_ID + " -name " + tempProperty + " -value " + tempValue

    # execute command
    if globalProps.bDEBUGFLAG == "True":
        print "DEBUG: tempCmd [" + tempCmd + "]"
        print "DEBUG: tempArgs [" + tempArgs + "]"
    
    p = subprocess.Popen([tempCmd, "-weburl", globalProps.UCDSERVER, "-authtoken", globalProps.AUTH_TOKEN, "setEnvironmentProperty", "-application", globalProps.APPLICATION_ID, "-environment", globalProps.ENVIRONMENT_ID, "-name", tempProperty, "-value", tempValue], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    tempOut, tempErr = p.communicate()
    
    if globalProps.bDEBUGFLAG == "True":
        print "DEBUG: tempOut [" + tempOut + "]"
        print "DEBUG: tempErr [" + tempErr + "]"   
    
    # remove trailing \t\r\n and whitespace
    tempOut = tempOut.rstrip()
    tempErr = tempErr.rstrip()    
    # ensure nothing went wrong
    checkStdErr(tempErr)



#########################################
#
# Function updateUCDEnvironmentProperty(tempProperty, tempValue)
#
# updateEnvironmentProperty info using the ucdclient api
#
# tempProperty format is elemnent1;element2;element3 etc
# where element is hostname:appList
#
#####################################

def updateUCDEnvironmentProperty(tempProperty, tempKey, tempValue):

    print "updating [" + tempProperty + "]"
    # get existing value
    existingValue = getEnvironmentProperty(tempProperty)
    
    if "Property not found" == existingValue:
        print "property does not exist, so creating it as new"
        newString = tempKey + ":" + tempValue
    
    else:
        if existingValue and existingValue.strip():
            print "property already exists [" + existingValue + "]"
            # - check for existing "hostname entry"
            #subPos = string.find(existingValue, tempKey)
            subPos = existingValue.find(tempKey)
            if str(subPos) == "-1":
                # "hostname entry" does not exist then add to it
                print "adding new value to existing values"
                newString = existingValue + ";" + tempKey + ":" + tempValue
            else:
                # "hostname entry" does exist so replace it
                print "replacing existing value"
                # - split into individual elements
                tempArray = str.split(existingValue, ";")
                newString = ""
                for element in tempArray:
                    # subPos = string.find(element, tempKey)
                    subPos = element.find(tempKey)
                    if str(subPos) == "-1":
                        # use the element value
                        newString = element + ";" + newString
                    else:
                        # use the new value
                        newString = tempKey + ":" + tempValue + ";" + newString
                # remove the extra ";" off the end
                newString = newString[:-1]
        else:
            print "property blank, treat it as if it never existed, so creating it as new"
            newString = tempKey + ":" + tempValue


    # update the environment property
    setUCDEnvironmentProperty(tempProperty, newString)


#########################################
#
# Function getServerResourceParentTags(resourceName)
#
#
#####################################

def getServerResourceParentTags(resourceName):

    resourceJSON = getResource(resourceName)
    if globalProps.bDEBUGFLAG == "True":
        print "JSON resource is"
        print resourceJSON
    
    jsonData = json.loads(resourceJSON)
    tagList = ""
    # extract TAGS from json
    
    parentValue = jsonData['parent']
    if isinstance(parentValue, dict):
        tagsValue = parentValue['tags']
        if isinstance(tagsValue, list):
            for tagData in tagsValue:
                if isinstance(tagData, dict):
                    tagName = tagData.get("name")
                    if globalProps.bDEBUGFLAG == "True":
                        print "tag [" + tagName + "]"
                    tagList = tagName + "," + tagList

        else:
            print "ERROR: JSON resource does not contain any [parent:tags] information"
            print "JSON resource is"
            print resourceJSON
    else:
        print "ERROR: JSON resource does not contain any [parent] information"
        print "JSON resource is"
        print resourceJSON

    # remove last comma
    tagList = tagList[:-1]
    print "tagList [" + tagList + "]"
    
    return tagList


#########################################
#
# Function checkTagRules(tagString, tempSeperator)
#
# RULES::

# 1. only accept a fixed list of tags
# 2. there must be WAS present for any group containing an APP
# 3. if LDX present there must also be JTS
# 4. if DB2 present, nothing else can be
# 5. if ORACLE present, nothing else can be
# 6. if IHS present nothing else can be
#
#####################################

def checkTagRules(tagString, tempSeperator):

    print "tagString [" + tagString + "]"
    # convert to a list of lowercase tags
    bRulesInValidated = "False"
    tagList = tagString.lower().split(tempSeperator)


    # 1. only accept a fixed list of tags
    # no longer accept "jrs"
    print "check valid tags rule"
    validList = ["was", "ihs", "db2", "oracle", "jts", "ccm", "qm", "rm", "rs", "dcc", "relm", "ldx", "gc", "lqe", "dm"]
    for tag in tagList:
        if tag in validList:
            print "-- " + tag + " in the valid list --> rule validated"
        else:
            print "-- ERROR: " + tag + " not in valid list --> rule invalidated"
            bRulesInValidated = "True"


    # 2. there must be WAS present for any group containing an APP
    # no longer accept "jrs"
    print "check WAS and {app} rule"
    appList = ["jts", "ccm", "qm", "rm", "rs", "dcc", "relm", "ldx", "gc", "lqe", "dm"]
    for tag in tagList:
        if tag in appList:
            if "was" in tagList:
                print "-- was and " + tag + " present --> rule validated"
            else:
                print "-- ERROR: " + tag + " present but no was --> rule invalidated"
                bRulesInValidated = "True"    
        else:
            print "-- " + tag + " not in applist --> no need to test"


    # 3. if LDX present there must also be JTS
    print "check LDX rule"
    if "ldx" in tagList:
        if "jts" in tagList:
            print "-- ldx and jts present --> rule validated"
        else:
            print "-- ERROR: ldx present but no jts --> rule invalidated"
            bRulesInValidated = "True"
    else:
        print "-- ldx not present --> no need to test"


    # 4. cannot have DB2 and any other TAG on the same server
    print "check DB2 database rule"
    if ("db2" in tagList):
        bLocalRuleFail="False"
        for tag in tagList:
            if tag != "db2":
                print "-- ERROR: " + tag + " and db2 cannot be on the same server --> rule invalidated"
                bRulesInValidated = "True"
                bLocalRuleFail = "True"
        if bLocalRuleFail == "False":
            print "-- only db2 present --> rule validated"
    else:
        print "-- db2 not present --> no need to test"


    # 5. cannot have ORACLE and any other TAG on the same server
    print "check ORACLE database rule"
    if ("oracle" in tagList):
        bLocalRuleFail="False"
        for tag in tagList:
            if tag != "oracle":
                print "-- ERROR: " + tag + " and oracle cannot be on the same server --> rule invalidated"
                bRulesInValidated = "True"
                bLocalRuleFail = "True"
        if bLocalRuleFail == "False":
            print "-- only oracle present --> rule validated"
    else:
        print "-- oracle not present --> no need to test"


    # 6. cannot have IHS and any other TAG on the same server
    print "check IHS rule"
    if ("ihs" in tagList):
        bLocalRuleFail="False"
        for tag in tagList:
            if tag != "ihs":
                print "-- ERROR: " + tag + " and ihs cannot be on the same server --> rule invalidated"
                bRulesInValidated = "True"
                bLocalRuleFail = "True"
        if bLocalRuleFail == "False":
            print "-- only ihs present --> rule validated"
    else:
        print "-- ihs not present --> no need to test"


    # confirm all is valid or exit
    if "True" == bRulesInValidated:
        # one or more rules invalid
        sys.exit(1)
    else:
        print "TAG list valid"



#########################################
#
# Function getAppsFromTags(tagString, tempSeperator)
# no longer accept "jrs"
#
#####################################

def getAppsFromTags(tagString, tempSeperator):

    print "tagString [" + tagString + "]"
    # convert to a list of lowercase tags
    tagList = tagString.lower().split(tempSeperator)
    removeList = ["was", "ihs", "db2", "oracle"]
    
    for a in removeList:
        if a in tagList:
            # remove tags that are not apps
            tagList.remove(a)

    # convert back into a string
    appString = tempSeperator.join(tagList)
    print "appString [" + appString + "]"

    return appString


#########################################
#
# Function getDatabaseFromTags(tagString, tempSeperator)
#
#
#####################################

def getDatabaseFromTags(tagString, tempSeperator):

    print "tagString [" + tagString + "]"
    # convert to a lowercase list
    tagList = tagString.lower().split(tempSeperator)
    databaseType =""
    
    # look for database entries
    if "db2" in tagList:
        databaseType = "db2"
    elif "oracle" in tagList:
        databaseType = "oracle"
    else:
        databaseType = "noDatabaseFound"
    
    print "databaseType [" + databaseType + "]"

    return databaseType



#########################################
#
# Function getWasRoleFromTags(tagString, tempSeperator)
#
#
#####################################

def getWasRoleFromTags(tagString, tempSeperator):

    print "tagString [" + tagString + "]"
    # convert to a lowercase list
    tagList = tagString.lower().split(tempSeperator)
    wasRole =""
    
    # look for database entries
    if "was" in tagList:
        wasRole = "was"
    elif "ihs" in tagList:
        wasRole = "ihs"
    else:
        wasRole = "noWasFound"
    
    print "wasRole [" + wasRole + "]"

    return wasRole

#########################################
#
# Function updateServerPropertiesFile(propFile, tmpKey, tmpValue)
#
# assumptions
# -- to search (match) for "tmpKey=" or "{whitespace}tmpKey="
# -- to replace / add a key with no leading whitespace
#
#####################################

def updateServerPropertiesFile(propFile, tmpKey, tmpValue):

    propFileCopy = propFile + ".ORIG"
    
    # remove leading white space from key
    matchKey = tmpKey.lstrip() + "="
    
    # python docs say "\n" will work on windows as well!
    newLine = matchKey + tmpValue + "\n"
    
    print "update property file [" + propFile + "]"
    if os.path.exists(propFile):
        print "file exists"
    else:
        print "ERROR: file does not exist"
        sys.exit(1)
    
    # copy propFile to propFileCopy
    print "copy to [" + propFileCopy + "]"
    shutil.copy(propFile, propFileCopy)
    if os.path.exists(propFileCopy):
        print "file exists"
    else:
        print "ERROR: file does not exist"
        sys.exit(1)    
    
    # read from propFileCopy and overwrite propFile
    fhIN = open(propFileCopy, "r")
    fhOUT = open(propFile, "w")
    bLineAdded = "False"
    
    for existingLine in fhIN:
        if globalProps.bDEBUGFLAG == "True":
            print "DEBUG: existingLine [" + existingLine.rstrip() + "]"
        # remove leading whitespace
        tempLine = existingLine.lstrip()
        subPos = tempLine.find(matchKey)
        if globalProps.bDEBUGFLAG == "True":
            print "DEBUG: subPos [" + str(subPos) + "]"
        if str(subPos) == "0":
            # replace the line
            print "replacing [" + existingLine.rstrip() + "] with [" + newLine.rstrip() + "]"
            fhOUT.write(newLine)
            bLineAdded = "True"
        else:
            # use the existing line
            fhOUT.write(existingLine)
    
    if bLineAdded == "False":
        # still have to add the line
        print "appending [" + newLine.rstrip() + "]"
        fhOUT.write(newLine)
    
    fhIN.close()
    fhOUT.close()

    # remove temporary file
    deleteFile(propFileCopy)


#####################################
#
# Function checkForUpperCaseChar(testString)
#
#
#####################################

def checkForUpperCaseChar(testString):

    rc = "False"
    for c in testString:
        if c.isupper():
            rc = "True"

    return rc

#####################################
#
# Function getAllEnvironmentPropertyNames()
#
#
#####################################

def getAllEnvironmentPropertyNames():

    resourceJSON = getAllEnvironmentProperties()
    if globalProps.bDEBUGFLAG == "True":
        print "JSON resource is"
        print resourceJSON
    
    jsonData = json.loads(resourceJSON)
    propertyList = ""
    # extract all properties from json
    
    for propertyData in jsonData:
        propertyName = propertyData.get("name")
        propertyList = propertyName + "," + propertyList

    # remove last comma
    propertyList = propertyList[:-1]
    print "propertyList [" + propertyList + "]"
    
    return propertyList

#####################################
#
# Function getAllEnvironmentPropertyValues()
#
#
#####################################

def getAllEnvironmentPropertyValues():

    propertyString = getAllEnvironmentPropertyNames()
    itemList = propertyString.split(seperatorString)
    for item in itemList:
        value = getEnvironmentProperty(item)
        print "[" + item + "] [" + value + "]"

#####################################
#
# Function wipeAllEnvironmentPropertyValues()
#
#
#####################################

def wipeAllEnvironmentPropertyValues():

    propertyString = getAllEnvironmentPropertyNames()
    itemList = propertyString.split(seperatorString)
    for item in itemList:
        value = getEnvironmentProperty(item)
        print "current [" + item + "] [" + value + "]"
        
        setUCDEnvironmentProperty(item, "")
        
        value = getEnvironmentProperty(item)
        print "NEW [" + item + "] [" + value + "]"

#####################################
#
# Function deleteFile(fileToDelete)
#
#
#####################################
def deleteFile(fileToDelete):

    print "deleting [" + fileToDelete + "]"
    if os.path.exists(fileToDelete):
        os.remove(fileToDelete)
        if os.path.exists(fileToDelete):
            print "ERROR: file does not exist"
            sys.exit(1)
        else:
            print "file deleted"
    else:
        print "ERROR: file does not exist"
        sys.exit(1)

#####################################
#
# Function deleteFileIfExists(fileToDelete)
#
#
#####################################
def deleteFileIfExists(fileToDelete):

    print "deleting [" + fileToDelete + "] if it exists"
    if os.path.exists(fileToDelete):
        deleteFile(fileToDelete)
    else:
        print "file does not exist, so no need to delete it"

#####################################
#
# Function createFile(fileToCreate)
#
#
#####################################
def createFile(fileToCreate):

    print "creating [" + fileToCreate + "]"
    open(fileToCreate, 'w').close()
    if os.path.exists(fileToCreate):
        print "file exists"
        
        if globalProps.bOSWINDOWS == "False":
            # if linux make it executable
            subprocess.Popen(["ls", "-la", fileToCreate])
            print "ensure file is executable"
            subprocess.Popen(["chmod", "u+x", fileToCreate])
            time.sleep(5)
            subprocess.Popen(["ls", "-la", fileToCreate])
        
    else:
        print "ERROR: file does not exist"
        sys.exit(1)


#####################################
#
# Function checkCustomLdapFileExists()
#
#
#####################################
def checkCustomLdapFileExists():
    
    ldapFolder = os.path.abspath(os.path.join(globalProps.SCRIPT_FOLDER, os.pardir))
    ldapFile = ldapFolder + "/CustomLDAP/Custom_LDAPSecurity.properties"
    print "checking [" + ldapFile + "] exists"
    if os.path.exists(ldapFile):
        print "file exists"
    else:
        print "ERROR: file does NOT exist"
        sys.exit(1)