#!/bin/bash

#********************************************************************************
# Licensed Materials - Property of IBM
# IBM Quick Deployer
# (c) Copyright IBM Corporation 2015. All Rights Reserved.
#
# U.S. Government Users Restricted Rights:
# Use, duplication or disclosure restricted by GSA ADP Schedule
# Contract with IBM Corp.
#
#********************************************************************************

#################################################################################
##
## Name: InstallMaintenanceFiles.sh
##
## Execute the script
## ------------------
## sh .\InstallMaintenanceFiles.sh
#
## assumptions
## -----------
## 1. there is already an install done [eg 6.0 GA]
## 2. nothing has been deployed into WAS
## 3. no inputs, control comes from $APPS (tagging system)
##
## this script does the following 
## ------------------------------
## for app = {jts, (csm, gc), ccm, rm, qm, dcc, relm}
## 1. create patch folder on {app} server [/JazzTeamServer/server/patch]
## 2. copy patch.zip to patch folder, overwrite existing if necessary
##
## for app = {ldx, rs, lqe}
## 1. get {app}.war files
## 2. overwrite existing $CLM_HOME/server/webapps/{app}.war files
##
## also adds release dependent misc files 
## eg for 6.0 if app = {gc}
## 1. get services.xml file
## 2. overwrite existing $CLM_HOME/server/conf/gc/services.xml file

##
#################################################################################


##############
## FUNCTIONS #
##############


## usage exitWithErrMsg tempString
exitWithErrMsg()
{
  tempString=$1
  echo "-- ERROR [${scriptName}] --"
  echo "-- message: ${tempString}"
  exit 1
}


## usage exitWithWarnMsg tempString
exitWithWarnMsg()
{
  tempString=$1
  echo "-- WARNING [${scriptName}] --"
  echo "-- message: ${tempString}"
  exit 0
}


## usage checkFileExists tempFile
checkFileExists()
{
  tempFile=$1
  if [ -f ${tempFile} ]; then
    echo "file [${tempFile}] exists"
  else
    errMsg="[${tempFile}] does not exist"
    exitWithErrMsg "${errMsg}"
  fi
}


## usage checkFolderExists tempFolder
checkFolderExists()
{
  tempFolder=$1
  if [ -d ${tempFolder} ]; then
    echo "folder [${tempFolder}] exists"
  else
    errMsg="[${tempFolder}] does not exist"
    exitWithErrMsg "${errMsg}"
  fi
}


## make a new folder including any missing folders in the path
makeFolderFullPath(){
  newFolder=$1
  echo "make new folder (inc path) [${newFolder}]"
  mkdir -p ${newFolder}
  checkFolderExists ${newFolder}
}


## usage download file from "jazzweb" to current folder
downloadHttpZip()
{
  fileUrl=$1
  filename=$(basename $fileUrl)

  sh $AUTO_PREP/MediaParameters/doWget.sh $fileUrl
  if [ $? == 0 ]
  then
    echo "download complete"
    checkFileExists $filename
  else
    errMsg="failed to download file ["${filename}"]"
    exitWithErrMsg "$errMsg"
  fi
}


# copy zipfile from "nfs" to current folder
copyZip()
{
  file=$1
  filename=$(basename $file)
  echo "copy ["${file}"] to ["${filename}"]"
  cp $file $filename
  if [ $? == 0 ]
  then
    echo "copy complete" 
    checkFileExists $filename
  else
    errMsg="failed to copy file ["${filename}"]"
    exitWithErrMsg "$errMsg"
  fi
}


## usage save existing war file and replace with new one
updateWithIfixFile()
{
  updateFile=$1
  updateFolder=$2
  targetFolder=$3
  saveInFolder=$4
  
  echo "save existing [${targetFolder}"/"${updateFile}] file in [${saveInFolder}"/"${updateFile}]"
  checkFileExists ${targetFolder}"/"${updateFile}
  cd ${targetFolder}
  tempUser=`ls -l ${updateFile} | cut -d' ' -f 3`
  tempGroup=`ls -l ${updateFile} | cut -d' ' -f 4`
  tempPermissions=`stat -c "%a" ${updateFile}`
  copyFile ${updateFile} ${saveInFolder}
  checkFileExists ${saveInFolder}"/"${updateFile}
  
  echo "delete existing [${targetFolder}"/"${updateFile}] file"
  deleteFile ${targetFolder} ${updateFile}
  
  echo "copying [${updateFolder}"/"${updateFile}] to [${targetFolder}]"
  checkFileExists $updateFolder"/"${updateFile}
  cd ${updateFolder}
  copyFile ${updateFile} ${targetFolder}
  checkFileExists ${targetFolder}"/"${updateFile}
  cd ${targetFolder}
  ls -la ${updateFile}
  echo "replace with original file [owner][group][permissions]"
  echo "[${tempUser}][${tempGroup}][${tempPermissions}]"
  chmod ${tempPermissions} ${updateFile}
  chgrp ${tempGroup} ${updateFile}
  chown ${tempUser} ${updateFile}
  ls -la ${updateFile}
  
}


## usage copyFile (force) from current folder to folder
copyFile()
{
  tempFile=$1
  tempFolder=$2
  echo "copy ["${tempFile}"] to ["${tempFolder}/${tempFile}"]"
  cp -f ${tempFile} ${tempFolder}/${tempFile}
  if [ $? == 0 ]; then
    echo "copy complete" 
    if [ -f ${tempFolder}/${tempFile} ]; then
      echo "copy successful, file exists"
    else
      errMsg="file does not exist"
      exitWithErrMsg "${errMsg}"
    fi
  else
    errMsg="copy failed"
    exitWithErrMsg "${errMsg}"
  fi
}


## usage deleteFile (force) from folder
deleteFile()
{
  tempFolder=$1
  tempFile=$2
  fileToDelete=${tempFolder}"/"${tempFile}
  
  if [ -f ${fileToDelete} ]; then
    echo "delete file ["${fileToDelete}"]"
    rm -f ${fileToDelete}
    if [ -f ${fileToDelete} ]; then
      errMsg="failed to delete file"
      exitWithErrMsg "${errMsg}"
    else
      echo "succesfully deleted [${fileToDelete}]"
    fi
  else
    echo "file [${fileToDelete}] does not exist, so cannot be deleted"
  fi
}


## usage recursively deleteFolder (force)
deleteFolder()
{
  tempFolder=$1
  
  if [ -d ${tempFolder} ]; then
    echo "delete folder ["${tempFolder}"]"
    rm -Rf ${tempFolder}
    if [ -d ${tempFolder} ]; then
      errMsg="failed to delete folder"
      exitWithErrMsg "${errMsg}"
    else
      echo "succesfully deleted [${tempFolder}]"
    fi
  else
    echo "folder [${tempFolder}] does not exist, so cannot be deleted"
  fi
}


## usage unzip contents to target folder
unzipZip()
{
  zipFile=$1
  targetDir=$2
  
  unzip ${zipFile} -d ${targetDir}
  if [ $? == 0 ]; then
    echo "unzip successful" 
  else
    errMsg="failed to unzip  ["${zipFile}"]"
    exitWithErrMsg "$errMsg"
  fi
}


## usage unzip partial contents to target folder
unzipZipPartial()
{
  zipFile=$1
  targetDir=$2
  sectionReference=$3
  
  unzip -j ${zipFile} ${sectionReference} -d ${targetDir}
  if [ $? == 0 ]; then
    echo "unzip successful" 
  else
    errMsg="failed to unzip section ["${sectionReference}"] from ["${zipFile}"]"
    exitWithErrMsg "$errMsg"
  fi
}


## calculate the application release eg 6.0.0
setAppRelease()
{
  # extract the prefix and then the release
  appMediaContent=`echo ${appMediaRoot} | cut -c -2`
  if [ "SS" == $appMediaContent ]; then
    # SSE
    appRelease=`echo ${appMediaRoot} | cut -c 4-`
  elif  [ "CL" == $appMediaContent ]; then
    # CLM
    appRelease=`echo ${appMediaRoot} | cut -c 4-`
  elif  [ "CE" == $appMediaContent ]; then
    appRelease=`echo ${appMediaRoot} | cut -c 3-`
  else
    errMsg="unsupported prefix ["${appMediaContent}"] in ["${appMediaRoot}"]"
    exitWithErrMsg "$errMsg"
  fi
  echo "appMediaRoot [${appMediaRoot}], appMediaContent [${appMediaContent}], appRelease [${appRelease}]"
}


## calculate the maintenance stream id eg 6.0.0.0
setMaintStreamID()
{
  # translate appRelease into the maintenance stream string
  tempAppRelease=$1
  
  case "$tempAppRelease" in
    602) maintClmStreamID="6.0.2.0"
       ;;
    601) maintClmStreamID="6.0.1.0"
       ;;
    600) maintClmStreamID="6.0.0.0"
       ;;
    60) maintClmStreamID="6.0.0.0"
       ;;
    *) errMsg="no maintenance stream for [${tempAppRelease}]"
       exitWithErrMsg "$errMsg"
       ;;
  esac
  
  echo "tempAppRelease [${tempAppRelease}], maintClmStreamID [${maintClmStreamID}]"
}



############
### MAIN ###
############

## assume we run from the script's directory containing
##  InstallMaintenenceFiles.sh
## use the following from env.properties
#   $CLM_BUILDROOTURL
#   $CLM_BUILD_ID
#   $AUTO_PREP

scriptName=$0
workingDir=`pwd -P`

# get env properties
envProp=/etc/env.properties
. $envProp

# get media properties
mediaProp=$AUTO_PREP/MediaParameters/media.properties
. $mediaProp

## use the following from media.pipeline
# patchZipSuffix=calm-patch-service-zip/output/native
# workbenchZipSuffix=calm-offering-zip/output/native


# target folders for ZIP and WAR files
CLM_HOME=/opt/IBM/JazzTeamServer
#localWorkbenchZipFolder=${CLM_HOME}"/UCDtemp"

localIFixZipFolder=${QD_InternalMedia}"/"${iFixMediaRoot}

localNewWebAppsFolder=${localIFixZipFolder}"/newWebApps"
localNewFilesFolder=${localIFixZipFolder}"/newFiles"
localPatchFileFolder=${localIFixZipFolder}"/newPatches"

patchZipTargetFolder=${CLM_HOME}/server/patch
warTargetFolder=${CLM_HOME}/server/webapps
savedWarFolder=$CLM_HOME/server/oldwebapps
savedMiscFilesFolder=$CLM_HOME/server/oldfiles


## ======================================================================
## input parameters
# none


## ======================================================================
## set ${appRelease} for later use
setAppRelease


## =======================================================
## set control flags for later use

installPatchZipFlag=FALSE
installWarFileFlag=FALSE
installMiscFilesFlag=FALSE
somethingToPatch=FALSE


validPatchZipApps="gc,dcc,relm,qm,rm,ccm,jts"
validWarFileApps="lqe,ldx,rs"
patchableApps=${validPatchZipApps}","${validWarFileApps}

apps=${APPS//,/ }
echo "apps [$apps]"

for a in $apps
do
  # set installPatchZipFlag is not release dependent 
  vpList=${validPatchZipApps//,/ }
  for vp in $vpList
  do
    if [[ $a == $vp ]]; then
      installPatchZipFlag=TRUE
      somethingToPatch=TRUE
    fi
  done

  # set installWarFileFlag is not release dependent
  vzList=${validWarFileApps//,/ }
  for vz in $vzList
  do
    if [[ $a == $vz ]]; then
      installWarFileFlag=TRUE
      somethingToPatch=TRUE
    fi
  done
  
  # set installMiscFilesFlag is release dependent
  
  if [ "60" == ${appRelease} ] || [ "600" == ${appRelease} ]; then
    if [[ $a == "gc" ]]; then
      installMiscFilesFlag=TRUE
      somethingToPatch=TRUE
    fi
  fi
  
done

echo "installPatchZipFlag = ["${installPatchZipFlag}"]"
echo "installWarFileFlag = ["${installWarFileFlag}"]"
echo "installMiscFilesFlag = ["${installMiscFilesFlag}"]"

# check there is something to patch
if [ ${somethingToPatch} == "FALSE" ]; then
  errMsg="not installing iFix as no application in [${apps}] matches one in [${patchableApps}]"
  exitWithWarnMsg "$errMsg"
fi



## =======================================================
## CHECK ##
## ========

if [ "NONE" == ${iFixVersion} ]; then
    ## do nothing
    echo "not installing iFix as $!iFixVersion [${iFixVersion}]"
    echo "-- EOF [${scriptName}] --"
    exit 0
fi


if [ "CLM" == $iFixMediaType ]; then
  echo "using [${iFixMediaType}] media location type"
  ## check build exists in env.properties
  if [ "X"$CLM_BUILD_ID"X" == "XX" ] || [ "X"$CLM_BUILDROOTURL"X" == "XX" ]; then
    errMsg="$!CLM_BUILDROOTURL [$CLM_BUILDROOTURL] or $!CLM_BUILD_ID [$CLM_BUILD_ID] not set!"
    exitWithErrMsg "$errMsg"
  else
    echo "using CLM repository, $!CLM_BUILDROOTURL [$CLM_BUILDROOTURL], $!CLM_BUILD_ID [$CLM_BUILD_ID]"
  fi

elif [ "ZIP" == $iFixMediaType ]; then 
  echo "using [${iFixMediaType}] media location type"
  
  if [ "FILE" == $mediaAccess ]; then
    ## iFixFile = ${iFixVersion}.zip
    ## using file ${mediaRoot}/${iFixMediaRoot}/${iFixVersion}.zip
    if [ "XX" == "X"${mediaRoot}"X" ] || [ "XX" == "X"${iFixMediaRoot}"X" ] || [ "XX" == "X"${iFixVersion}"X" ]; then
      errMsg="$!mediaRoot [$mediaRoot] or $!iFixMediaRoot [$iFixMediaRoot] or $!iFixVersion [$iFixVersion] not set!"
      exitWithErrMsg "$errMsg"
    else
      iFixFile=${iFixVersion}.zip
      iFixZipFileLocation=${mediaRoot}"/"${iFixMediaRoot}"/"${iFixFile}
      echo "iFix file, $!iFixZipFileLocation [${iFixZipFileLocation}]"
      
      if [ "X"$nfsEnabled"X" == "XX" ]; then
        echo "no NFS drive exists, assuming any ZIP files are local"
        nfsEnabled=FALSE
      else
        # need to check value
        if [ "FALSE" == $nfsEnabled ]; then
          echo "no NFS drive exists, assuming any ZIP files are local"
        elif [ "TRUE" == $nfsEnabled ]; then 
          echo "NFS drive exists, assuming any ZIP files are remote"
        else
          errMsg="["${nfsEnabled}"] nfsEnabled not supported"
          exitWithErrMsg "$errMsg"
        fi
      fi
     fi
  else
    errMsg="$!mediaAccess [$mediaAccess] not supported!"
    exitWithErrMsg "$errMsg"
  fi

else
  errMsg="["${iFixMediaType}"] media location type not supported"
  exitWithErrMsg "$errMsg"
fi



## =======================================================
## COMMON FOLDERS ##
## ===========
# as all input checks complete its now possible to create the download folders to contain the unzip parts
echo "delete any old iFix downloads"
deleteFolder ${localIFixZipFolder}
echo "create new iFix download folders"
makeFolderFullPath ${localIFixZipFolder}
makeFolderFullPath ${localPatchFileFolder}
makeFolderFullPath ${localNewWebAppsFolder}
makeFolderFullPath ${localNewFilesFolder}


## =======================================================
## DOWNLOAD ##
## ===========


if [ "CLM" == $iFixMediaType ]; then 

  if [ "TRUE" == ${installPatchZipFlag} ]; then 
    ## construct location of patch zipfile
    # - first set maintClmStreamID using setMaintStreamID
    setMaintStreamID $appRelease
    patchZipFileName="CLM_server_patch_"${maintClmStreamID}"-"${CLM_BUILD_ID}".zip"
    patchZipFolder=${CLM_BUILDROOTURL}"/"${patchZipSuffix}
    patchZipFile=${patchZipFolder}"/"${patchZipFileName}
    echo "CLM patch zip file (url) ["${patchZipFile}"]"

    echo "==== downloading iFix patch zip file ===="
    ## download patch ZIP file to ${localPatchFileFolder}
    cd ${localPatchFileFolder}
    downloadHttpZip ${patchZipFile}
    ls -la
  else
    echo "no need to download iFix patch zip file"
  fi
  
  if [ "TRUE" == ${installWarFileFlag} ] || [ "TRUE" == ${installMiscFilesFlag} ]; then
    echo "==== downloading workbench zip file ===="
    
    ## construct location of workbench zipfile
    # - first set maintClmStreamID using setMaintStreamID
    setMaintStreamID $appRelease
    workbenchZipFileName="Workbench-for-CLM-Linux64_"${maintClmStreamID}"-"${CLM_BUILD_ID}".zip"
    workbenchZipFolder=${CLM_BUILDROOTURL}"/"${workbenchZipSuffix}
    workbenchZipFile=${workbenchZipFolder}"/"${workbenchZipFileName}
    echo "CLM workbench zip file ["${workbenchZipFile}"]"
    
    ## download workbench ZIP file to ${localIFixZipFolder}
    echo "download workbench zip file"
    timestamp=`date +%s`
    echo "DEBUG (ready to get ZIP) : current epoch time is ["${timestamp}"] Sec"
    cd ${localIFixZipFolder}
    downloadHttpZip ${workbenchZipFile}
    ls -la
    
    timestamp=`date +%s`
    echo "DEBUG (ready to unzip ZIP) : current epoch time is ["${timestamp}"] Sec"
    
    # if required just unzip the war files from the workbench ZIP file
    if [ "TRUE" == ${installWarFileFlag} ]; then
      echo "just unzipping the war files"
      # unzip -j ${workbenchZipFileName} server/tomcat/webapps/*.war -d ${localNewWebAppsFolder}
      # unzipZipPartial ${workbenchZipFileName} ${localNewWebAppsFolder} server/tomcat/webapps/*.war
      # set the war file location in the workbench
      if [ "60" == ${appRelease} ] || [ "600" == ${appRelease} ]; then
        workbenchWarFolder=server/tomcat/webapps
      else
        workbenchWarFolder=server/webapps
      fi
      echo "workbenchWarFolder = [${workbenchWarFolder}]"
      unzipZipPartial ${workbenchZipFileName} ${localNewWebAppsFolder} ${workbenchWarFolder}/*.war
      
      ls -la ${localNewWebAppsFolder}
    fi
    
    # if required just unzip the misc files from the workbench ZIP file
    if [ "TRUE" == ${installMiscFilesFlag} ]; then
      
      if [ "60" == ${appRelease} ] || [ "600" == ${appRelease} ]; then
        echo "just unzipping /server/conf/gc/services.xml"
        # unzip -j ${workbenchZipFileName} server/conf/gc/services.xml -d ${localNewFilesFolder}
        unzipZipPartial ${workbenchZipFileName} ${localNewFilesFolder} server/conf/gc/services.xml
      fi
      
      ls -la ${localNewFilesFolder}
    fi
  
    timestamp=`date +%s`
    echo "DEBUG (ZIP unzipped) : current epoch time is ["${timestamp}"] Sec"
  
  else
    echo "no need to download workbench zip file"
  fi
  
fi


if [ "ZIP" == $iFixMediaType ]; then
  ## selectively extract to ${localNewWebAppsFolder} ${localNewFilesFolder} ${localPatchFileFolder}

  timestamp=`date +%s`
  echo "DEBUG (ready to get ZIP) : current epoch time is ["${timestamp}"] Sec"
  # only download a copy if NFS has been setup
  if [ "TRUE" == ${nfsEnabled} ]; then
    # copy the zip
    cd ${localIFixZipFolder}
    copyZip ${iFixZipFileLocation}
    ls -la
    iFixFilePath=${localIFixZipFolder}"/"${iFixFile}
    zipNeedsDeleted=TRUE
  else
    # will use the file direct from its "local location"
    iFixFilePath=${iFixZipFileLocation}
  fi
  
  timestamp=`date +%s`
  echo "DEBUG (ready to unzip ZIP) : current epoch time is ["${timestamp}"] Sec"
  
  # just do partial unzips from the iFix ZIP file
  
  if [ "TRUE" == ${installPatchZipFlag} ]; then 
    echo "just unzipping server patch zipfile"
    unzipZipPartial ${iFixFilePath} ${localPatchFileFolder} CLM_server_patch*.zip
    ls -la ${localPatchFileFolder}
    patchZipFileName=`ls ${localPatchFileFolder} | grep CLM_server_patch`
    echo "$!patchZipFileName [${patchZipFileName}]"
  fi
  
  if [ "TRUE" == ${installWarFileFlag} ]; then
    echo "just unzipping the war files"
    unzipZipPartial ${iFixFilePath} ${localNewWebAppsFolder} *.war
    ls -la ${localNewWebAppsFolder}
  fi
  
  if [ "TRUE" == ${installMiscFilesFlag} ]; then
    # every release will have a different set of misc files
    
    if [ "60" == ${appRelease} ] || [ "600" == ${appRelease} ]; then
      echo "just unzipping [gc] services.xml"
      unzipZipPartial ${iFixFilePath} ${localNewFilesFolder} services.xml
    fi
    ls -la ${localNewFilesFolder}
    
  fi
  
  timestamp=`date +%s`
  echo "DEBUG (ZIP unzipped) : current epoch time is ["${timestamp}"] Sec"
fi



## =======================================================
## INSTALL ##
## ==========

# ==== only use patch zip file if appropriate ====
if [ "TRUE" == ${installPatchZipFlag} ]; then 
  echo "==== updating iFix patch zip file ===="
  
  # copy zip to target folder, creating folder if it does not already exist
  makeFolderFullPath  ${patchZipTargetFolder}
  echo "delete existing [$patchZipTargetFolder"/"${patchZipFileName}] file"
  deleteFile ${patchZipTargetFolder} ${patchZipFileName}
  cd ${localPatchFileFolder}
  copyFile ${patchZipFileName} ${patchZipTargetFolder}
  checkFileExists $patchZipTargetFolder"/"${patchZipFileName}
  ls -la ${patchZipTargetFolder}
  
else
  echo "no need to install iFix patch zip file"
fi


# ==== only use war files if appropriate ====
if [ "TRUE" == ${installWarFileFlag} ]; then
  echo "updating application war files"
  
  # make safe folder for old war files
  makeFolderFullPath  ${savedWarFolder}
  
  for a in $apps
  do  
    if [[ $a == "ldx" ]]; then
      updateWithIfixFile ldx.war ${localNewWebAppsFolder} ${warTargetFolder} ${savedWarFolder}
    fi
    if [[ $a == "lqe" ]]; then
      updateWithIfixFile lqe.war ${localNewWebAppsFolder} ${warTargetFolder} ${savedWarFolder}
    fi 
    if [[ $a == "rs" ]]; then
      updateWithIfixFile rs.war ${localNewWebAppsFolder} ${warTargetFolder} ${savedWarFolder}
    fi  
  done
  
echo "saved war files ["${savedWarFolder}"]"
  ls -la ${savedWarFolder}
echo "updated war files ["${warTargetFolder}"]"
  ls -la ${warTargetFolder}

fi


# ==== only use misc files if appropriate ====
if [ "TRUE" == ${installMiscFilesFlag} ]; then
  echo "updating miscelaneous files"

  # make safe folder for old misc files
  makeFolderFullPath  ${savedMiscFilesFolder}
  
  for a in $apps
  do
  
    if [ "60" == ${appRelease} ] || [ "600" == ${appRelease} ]; then
      if [[ $a == "gc" ]]; then
        # since 60iFix003 need to update services.xml
        gcConfFolder=${CLM_HOME}/server/conf/gc
        updateWithIfixFile services.xml ${localNewFilesFolder} ${gcConfFolder} ${savedMiscFilesFolder}
        echo "updated folder ["${gcConfFolder}"]"
        ls -la ${gcConfFolder}
      fi
    fi
  
  done
  
  echo "saved misc files ["${savedMiscFilesFolder}"]"
  ls -la ${savedMiscFilesFolder}
  
fi



## =======================================================
## TIDY UP ##
## ==========
echo "delete any iFix downloads folders"
deleteFolder ${localIFixZipFolder}



## =======================================================
## === finished ====
echo "-- EOF [${scriptName}] SUCCESS --"
exit 0