#!/bin/bash
#********************************************************************************
# Licensed Materials - Property of IBM
# (c) Copyright IBM Corporation 2015. All Rights Reserved.
#  
# U.S. Government Users Restricted Rights:
# Use, duplication or disclosure restricted by GSA ADP Schedule
# Contract with IBM Corp.
#
#********************************************************************************


#############
## FUNCTIONS 
#############

setDefaultInputs()
{
  USERNAME="defaultUSERNAME"
  USERPASSWORD="defaultUSERPASSWORD"
  UCDSERVER="defaultUCDSERVER"
}


checkDefaultInputs()
{
  if [ "$USERNAME" == "defaultUSERNAME" ]; then
    errMsg="USERNAME [$USERNAME] still contains its default value"
    exitWithErrMsg "$errMsg"
  fi
  if [ "$USERPASSWORD" == "defaultUSERPASSWORD" ]; then
    errMsg="USERPASSWORD [$USERPASSWORD] still contains its default value"
    exitWithErrMsg "$errMsg"
  fi
  if [ "$UCDSERVER" == "defaultUCDSERVER" ]; then
    errMsg="UCDSERVER [$UCDSERVER] still contains its default value"
    exitWithErrMsg "$errMsg"
  fi
}


exitWithErrMsg()
{
  tempString=$1
  echo "-- ERROR [${scriptName}] --"
  echo "-- message: $tempString"
  exit 1
}


checkFileExists()
{
  tempFile=$1
  if [ -f $tempFile ]; then
    echo "  file [$tempFile] exists"
  else
    errMsg="file [$tempFile] does not exist"
    exitWithErrMsg "$errMsg"
  fi
}


checkFolderExists()
{
  tempFolder=$1
  if [ -d $tempFolder ]; then
    echo "  folder [$tempFolder] exists"
  else
    errMsg="folder [$tempFolder] does not exist"
    exitWithErrMsg "$errMsg"
  fi
}


# will assume these are globals
# $USERNAME
# $USERPASSWORD
# $UCDSERVER
# $UCDCLIEXE
addTempResource()
{

  tempResourceFile=$1
  
  echo "  creating temporary resource defined in ["${tempResourceFile}"]"
  COMMAND=${UCDCLIEXE}" -username "${USERNAME}" -password "${USERPASSWORD}" -weburl "${UCDSERVER}" createResource "${tempResourceFile}
  # echo COMMAND: [${COMMAND}]
  # execute the command
  # -- success and ucd returns a json representation of the resource
  # -- failure and no json is returned
  $COMMAND
  errCode=$?
  if [ $errCode == 0 ]; then
    echo "  successfully created resource" 
  else
    errMsg="[$errCode] failed to create resource"
    exitWithErrMsg "$errMsg"
  fi
}


# will assume these are globals
# $USERNAME
# $USERPASSWORD
# $UCDSERVER
# $UCDCLIEXE
removeTempResource()
{
  tempResource=$1
  
  echo "  deleting resource  ["${tempResource}"]"
  COMMAND=${UCDCLIEXE}" -username "${USERNAME}" -password "${USERPASSWORD}" -weburl "${UCDSERVER}" deleteResource -resource "${tempResource}
  # echo COMMAND: [${COMMAND}]
  # execute the command
  $COMMAND
  errCode=$?
  if [ $errCode == 0 ]; then
    echo "  successfully deleteded resource"
  else
    errMsg="[$errCode] failed to delete resource"
    exitWithErrMsg "$errMsg"
  fi
}

# will assume these are globals
# $USERNAME
# $USERPASSWORD
# $UCDSERVER
# $UCDCLIEXE
addTagToResource()
{

  resourceName=$1
  tagName=$2
  tagColour=$3
  
  echo "  adding tag ["${tagName}"] with colour ["${tagColour}"] to resource ["${resourceName}"]"
  COMMAND=${UCDCLIEXE}" -username "${USERNAME}" -password "${USERPASSWORD}" -weburl "${UCDSERVER}" addTagToResource -resource "${resourceName}" -tag "${tagName}" -color "${tagColour}
  # echo COMMAND: [${COMMAND}]
  # execute the command
  $COMMAND
  errCode=$?
  if [ $errCode == 0 ]; then
    echo "  successfully added tag"
  else
    errMsg="[$errCode] failed to add tag"
    exitWithErrMsg "$errMsg"
  fi
}


####################
##                ##
## ==== MAIN ==== ##
##                ##
####################

# UCDCLIEXE="./udclient"
TEMPRESOURCEFILE=tagsCreate.json
TEMPRESOURCENAME="/rational-qd-tag-resource"


wasColour=9932CC
ihsColour=$wasColour
db2Colour=000000
appColour=DDA0DD

workingDir=`pwd -P`
scriptName=$0


# -- set default values
echo "setting default inputs values"
setDefaultInputs

# -- process inputs
echo "reading inputs values"
if [ $# != 3 ]; then
  echo "ERROR: script "${scriptName}" requires 3 arguments: USERNAME=<UserName> USERPASSWORD=<UserPassword> UCDSERVER=<UCDURL ex:https://UCDHostName:8443>"
  exit 1
fi

while [[ $# > 0 ]]; do
   key=`echo $1 | cut -d'=' -f 1`
   value=`echo $1 | cut -d'=' -f 1 --complement`
   echo "[$key] [$value]"
   if [ "USERNAME"  == "$key" ]; then
      USERNAME=$value
   fi
   if [ "USERPASSWORD"  == "$key" ]; then
      USERPASSWORD=$value
   fi
   if [ "UCDSERVER"  == "$key" ]; then
      UCDSERVER=$value
   fi

   shift
done

# -- check all the defaults changed
echo "checking default inputs values are updated"
checkDefaultInputs

# -- ucd cli client needs JAVA_HOME set, so check it exists
echo "JAVA_HOME currently [${JAVA_HOME}]"
if [ "" == "${JAVA_HOME}" ]; then
  errMsg="You must have JAVA_HOME set (exported) on your environment to use the uDeploy client."
  exitWithErrMsg "$errMsg"
else
  echo "check [${JAVA_HOME}] folder exists"
  checkFolderExists ${JAVA_HOME}
fi

# -- check the ucd cli client exists
echo "check the ucd cli client exists"
echo "UCDCLIEXE currently [${UCDCLIEXE}]"
if [ "" == "${UCDCLIEXE}" ]; then
  errMsg="You must have the location of the ucd cli client [UCDCLIEXE] set (exported) on your environment."
  exitWithErrMsg "$errMsg"
else
  echo "check [${UCDCLIEXE}] file exists"
  checkFileExists ${UCDCLIEXE}
fi

# -- check the resource json file exists
echo "check the resource json file exists"
checkFileExists $TEMPRESOURCEFILE

# -- create a temporary resource
echo "create a temporary resource"
addTempResource $TEMPRESOURCEFILE

# -- add tags to the resource (and therefore to all of UCD)
echo "add tags to the resource (and therefore to all of UCD)"
WASLIST="WAS IHS"
for tag in $WASLIST
do
  echo "  - $tag"
  addTagToResource $TEMPRESOURCENAME $tag $wasColour
done

echo "  - DB2"
addTagToResource $TEMPRESOURCENAME "DB2" $db2Colour


APPLIST="JTS CCM QM RM DCC LDX GC RELM RS LQE DM"
for tag in $APPLIST
do
  echo "  - $tag"
  addTagToResource $TEMPRESOURCENAME $tag $appColour
done


# -- delete the temmporary resource
echo "delete the temporary resource"
removeTempResource $TEMPRESOURCENAME


# -- end --
echo "end of file [${scriptName}]"
