#!/bin/bash
#********************************************************************************
# Licensed Materials - Property of IBM
# (c) Copyright IBM Corporation 2015. All Rights Reserved.
#  
# U.S. Government Users Restricted Rights:
# Use, duplication or disclosure restricted by GSA ADP Schedule
# Contract with IBM Corp.
#
#********************************************************************************


#############
## FUNCTIONS 
#############

setDefaultInputs()
{
  APPLICATIONVERSION="defaultAPPLICATIONVERSION"
  COMPONENTVERSION="defaultCOMPONENTVERSION"
  BASEFOLDER="defaultBASEFOLDER"
}


checkDefaultInputs()
{
  if [ "$APPLICATIONVERSION" == "defaultAPPLICATIONVERSION" ]; then
    errMsg="APPLICATIONVERSION [$APPLICATIONVERSION] still contains its default value"
    exitWithErrMsg "$errMsg"
  fi
  if [ "$COMPONENTVERSION" == "defaultCOMPONENTVERSION" ]; then
    errMsg="COMPONENTVERSION [$COMPONENTVERSION] still contains its default value"
    exitWithErrMsg "$errMsg"
  fi
  if [ "$BASEFOLDER" == "defaultBASEFOLDER" ]; then
    errMsg="BASEFOLDER [$BASEFOLDER] still contains its default value"
    exitWithErrMsg "$errMsg"
  fi
}


exitWithErrMsg()
{
  tempString=$1
  echo "-- ERROR [${scriptName}] --"
  echo "-- message: $tempString"
  exit 1
}


checkFileExists()
{
  tempFile=$1
  if [ -f $tempFile ]; then
    echo "  file [$tempFile] exists"
  else
    errMsg="file [$tempFile] does not exist"
    exitWithErrMsg "$errMsg"
  fi
}


checkFolderExists()
{
  tempFolder=$1
  if [ -d $tempFolder ]; then
    echo "  folder [$tempFolder] exists"
  else
    errMsg="folder [$tempFolder] does not exist"
    exitWithErrMsg "$errMsg"
  fi
}


unzipZip()
{
  zipFile=$1
  targetDir=$2
  
  unzip ${zipFile} -d ${targetDir}
  if [ $? == 0 ]; then
    echo "  unzip successful" 
  else
    errMsg="failed to unzip  ["${zipFile}"]"
    exitWithErrMsg "$errMsg"
  fi
}


isFolderEmpty()
{
  # folder=$1
  tempFileCount=`ls $1 | wc -w`
  if [ $tempFileCount -eq 0 ]; then
    echo "folder [$1] is empty"
  else
    echo "contents of [$1]"
    ls $1
    errMsg="expected [$1] to be empty, found [$tempFileCount] files"
    exitWithErrMsg "$errMsg"
  fi
}

####################
##                ##
## ==== MAIN ==== ##
##                ##
####################


scriptName=$0
workingDir=`pwd -P`

# -- set default values
echo "setting default inputs values"
setDefaultInputs

# -- process inputs
echo "reading inputs values"
if [ $# != 3 ]; then
  echo "ERROR: script "${scriptName}" requires 3 arguments: BASEFOLDER=<base folder for unzipping the scripts> APPLICATIONVERSION=<ApplicationVersion> COMPONENTVERSION=<ComponentVersion>"
  exit 1
fi

while [[ $# > 0 ]]; do
   key=`echo $1 | cut -d'=' -f 1`
   value=`echo $1 | cut -d'=' -f 1 --complement`
   echo "[$key] [$value]"
   if [ "APPLICATIONVERSION"  == "$key" ]; then
      APPLICATIONVERSION=$value
   fi
   if [ "COMPONENTVERSION"  == "$key" ]; then
      COMPONENTVERSION=$value
   fi
   if [ "BASEFOLDER"  == "$key" ]; then
      BASEFOLDER=$value
   fi

   shift
done

# -- check all the defaults changed
echo "checking default inputs values are updated"
checkDefaultInputs

# -- check folder to contain the unzipped component(s) scripts exists
mkdir -p ${BASEFOLDER}
echo "Check ["${BASEFOLDER}"] exists"
checkFolderExists ${BASEFOLDER}

LIST="Application ApplicationServer Database InstallationManager SystemPre-Requisite"

# (pre)check that all required zip files exist
for Component in $LIST
do
  ZIPFILE="Rational_QD_"${Component}"_"${APPLICATIONVERSION}"_*.zip"  
  echo "Check ["${ZIPFILE}"] exists"
  checkFileExists ${ZIPFILE}
done

# (pre) check that none of the target folders are populated
for Component in $LIST
do
  FOLDER=${BASEFOLDER}"/"${APPLICATIONVERSION}"/"${COMPONENTVERSION}"/"${Component}
  echo "ensure target folder exists"
  mkdir -p ${FOLDER}
  checkFolderExists ${FOLDER}
  
  echo "confirm target folder is empty"
  isFolderEmpty ${FOLDER}
  
done


# unzip to the target folders
for Component in $LIST
do

  ZIPFILE="Rational_QD_"${Component}"_"${APPLICATIONVERSION}"_*.zip"
  FOLDER=${BASEFOLDER}"/"${APPLICATIONVERSION}"/"${COMPONENTVERSION}"/"${Component}
  echo "Unzip ["${ZIPFILE}"] into folder ["${FOLDER}"]"
  unzipZip ${ZIPFILE} ${FOLDER}
  
done

echo "SUCCESS [${scriptName}]: Quick Deployer scripts successfully unzipped"
exit 0
